# 
# The Button widget
# 
# This is a very simple widget. It draws a small line of text, at
# the set row/column position, with some form of differentiation
# for whether the button is in focus or not. Its input processor
# should do something interesting (inherit from this, or add a callback
# to the class) when the space bar is hit.
#

class ch_button { 
	inherit chwidget

	public method input { ichar } {
		set s [ get_screen ] 

		if { $ichar == " " } {
			$s debug_go "PRESSED BUTTON: $_label"
		}

		return 0 
	}

	public method onfocus { } { 
		set _focussed 1
		draw_on_focus 
	}  

	public method unfocus { } { 
		set _focussed 0
		draw_off_focus 
	} 

	public method draw { } {
		set s [ get_screen ]

		if { $_focussed == 1 } {
			draw_on_focus
		} else {
			draw_off_focus 
		}
	}

	public method draw_on_focus { } 
	public method draw_off_focus { } 

	private variable _rpos {}
	private variable _cpos {}
	private variable _label {}
	private variable _focussed 0 

	public method set_rpos { pos } {
		set _rpos $pos
	}

	public method set_cpos { pos } {
		set _cpos $pos
	}

	public constructor { label } {
		set _label $label
	}

}

body ch_button::draw_on_focus { } {

	if { $_rpos == {} } {
		# can't draw yet, haven't been placed

		return
	}

	set s [ get_screen ] 

	$s on_bold
	$s draw_text $_rpos $_cpos $_label
	$s off_bold
}

body ch_button::draw_off_focus { } {
	global n

	if {! [ info exists n ] } {
		set n 0
	}

	if { $_rpos == {} } {
		set s [ get_screen ] 
		if { $s != {} } {
			incr n 
		}

		# can't draw yet, haven't been placed

		return
	}

	set s [ get_screen ] 
	$s draw_text $_rpos $_cpos $_label
	
}

