#
# The Screen Widget
#
# This should be the first widget created in an application (although
# it doesn't absolutely have to). If no screen context is specified to
# the constructor, one will be created and initialized for you; as soon
# as you create the Screen widget, your application has raw control over
# the terminal. 
#
# The Screen widget should contain (via calls to "$this add") all of the
# displayed widgets on the screen.
#
# You give Screen control over input by calling "input_loop". You can
# implement customized input loop behavior by obtaining the Screen's
# screen context and using it (or something else) to grab input, and
# passing that input to the Screen's "input" method, ie:
#
# > set screen [ ch_screen #auto ] 
# > $screen input_loop
#
# ... is equivalent to:
#
# > set screen [ ch_screen #auto ]
# > set gc [ ch_screen get_screen ]
# > while { 1 } { 
# > 	$gc paint
# > 	$screen input [ $gc getchar ] 
# > }
#
# Screen catches two special input cases --- the application abort key
# (defaulting to ESC) and the widget focus switch key (defaulting to 
# TAB). Any other input is passed through to the currently focussed
# child.
#
#

class ch_screen { 
	inherit chwidget 

	private variable _rotchar "\x09"
	private variable _abtchar "\x1b"

	public method input_loop { }

	public method set_rfocuschar { rchar } { set _rotchar $rchar }
	public method get_rfocuschar { } { return $_rotchar } 
	public method set_abortchar { rchar } { set _abtchar $rchar }
	public method get_abortchar { } { return $_abtchar } 

	public method input { ichar } 

	public constructor { { screen {} } } {
		if { $screen != {} } {
			set_screen $screen 
		} else {
			set s [ charoo ::#auto ] 
			set_screen $s
		}
	}

}

body ch_screen::input { ichar } { 

	if { [ get_childcount ] == 0 } {
		return 0
	} 

	if { $ichar == $_rotchar } {
		rot_focus
		return 0
	} 

	if { $ichar == $_abtchar } { 
		exit 0 
	}

	set f [ get_focus ] 
	set r [ $f input $ichar ]

	if { $r == 1 } {
		$this del [ get_focusid ] 
	}

}

body ch_screen::input_loop { } {
	set gc [ get_screen ] 

	while { 1 } {
		$gc paint
		set c [ $gc getchar ] 
		input $c
	}

}
