# 
# Text drawing primitives. These are useable by themselves, 
# apart from the widget stuff. This is basically a simple 
# layer over Danny's tclterm. It's fairly self-explanatory.
# 
# Sorry that functions are taking y,x instead of x,y. Ohwell.
#

class charoo { 
	
	public method draw_textbox { rpos cpos text }
	public method draw_blankbox { rpos cpos rsize csize } 
	public method draw_box { rpos cpos rsize csize } 

	public method draw_textbox_c { text } 
	public method draw_blankbox_c { rsize csize } 
	public method draw_box_c { rsize csize } 

	public method draw_pgraph { rpos cpos text } 
	public method draw_text { rpos cpos text } 

	public method draw_hregion { rpos { direction "up" } } 
	public method draw_vregion { cpos { direction "left" } } 

	public method draw_hrule { rpos } 
	public method draw_vrule { cpos } 

	public method draw_line_h { rpos cpos len { corner {} } } 	
	public method draw_line_v { rpos cpos len { corner {} } } 

	public method clear_textbox { rpos cpos text }
	public method clear_blankbox { rpos cpos rsize csize } 
	public method clear_box { rpos cpos rsize csize } 

	public method clear_line_h { rpos cpos len }
	public method clear_line_v { rpos cpos len } 

	public method clear_hrule  { rpos } 
	public method clear_vrule  { cpos }

	public method on_bold { } 
	public method off_bold { } 

	public method pause { }

	public method getchar { } 

	public method debug { msg } 
	public method debug_go { msg } 

	public method paint { } {
		term corner
	}

	public method suspend { } {
		term clear
		term corner
		term exit
	}

	private variable _rows
	private variable _cols

	private variable _hledge "-"
	private variable _vledge "|"
	private variable _corner "+"
	private variable _clear  " " 

	public method set_hedge { hchar }  { set _hledge $hchar }
	public method set_vedge { vchar }  { set _vledge $vchar }
	public method set_corner { cchar } { set _corner $cchar }
	public method set_clear { cchar } { set _clear $cchar } 
	public method get_hedge { } { return $_hledge } 
	public method get_vedge { } { return $_vledge } 
	public method get_corner { } { return $_corner } 
	public method get_clear { } { return $_clear } 

	public method get_rows { } { return $_rows } 
	public method get_cols { } { return $_cols } 

	private variable _boxrpos  
	private variable _boxcpos 

	public constructor {} { 
		term init raw
		term enter
		term clear
		term corner

		set _rows [ term rows ] 
		set _cols [ term cols ] 
	}

	public destructor {
		term exit
	}
}

body charoo::debug { msg } {
	term move $_rows $_cols 
	term puts $msg

	term corner
	term getchar
}

body charoo::debug_go { msg } {
	on_bold
	term move $_rows $_cols 
	term puts $msg
	off_bold
}

body charoo::draw_line_h { rpos cpos len { corner {} } } {

	if { $rpos > $_rows || [ expr $cpos + $len ] > $_cols } {
		return "line too long for screen"
	}

	term move $rpos $cpos

	for { set i 0 } { $i < $len } { incr i } { 
		if { $i == 0 || $i == [ expr $len - 1 ] } {
			if { $corner != {} } {
				term puts $_corner
			} else { 
				term puts $_hledge 
			}
		} else { 
			term puts $_hledge
		}
	}

	return {}
} 

body charoo::draw_line_v { rpos cpos len { corner {} } } {

	if { [expr $rpos + $len ] > $_rows || $cpos > $_cols } {
		return "line too long for screen"
	}

	term move $rpos $cpos
	
	for { set i 0 } { $i < $len } { incr i } { 
		term move [ expr $rpos + $i ] $cpos	

		if { $i == 0 || $i == [ expr $len - 1 ] } {
			if { $corner != {} } {
				term puts $_corner
			} else { 
				term puts $_vledge
			}
		} else { 
			term puts $_vledge
		}
	}

	return {}
} 

body charoo::draw_box_c { rsize csize } {
	
	set srow [ expr $_rows - $rsize ]
	set srow [ expr $srow / 2 ] 

	set scol [ expr $_cols - $csize ]
	set scol [ expr $scol / 2 ] 

	draw_box $srow $scol $rsize $csize

	return {}
}

body charoo::draw_blankbox_c { rsize csize } { 

	set srow [ expr $_rows - $rsize ]
	set srow [ expr $srow / 2 ] 

	set scol [ expr $_cols - $csize ]
	set scol [ expr $scol / 2 ] 

	draw_blankbox $srow $scol $rsize $csize

	return {}	
}

body charoo::draw_hrule { rpos } {

	if { $rpos > $_rows } {
		return "hrule too big for screen"
	}

	draw_line_h $rpos 0 $_cols

	return {}
}

body charoo::draw_vrule { cpos } {
	
	if { $cpos > $_cols } { 
		return "vrule too big for screen"
	}

	draw_line_v 0 $cpos $_rows

	return {}
}

body charoo::draw_hregion { rpos { direction "up" } } {
	if { $rpos > $_rows } {
		return "hrule too big for screen"
	}

	draw_hrule $rpos

	if { $direction == "up" } {
		for { set i [ expr $rpos - 1 ]  } { $i > 0 } { incr i -1 } {
			clear_hrule $i
		}
	} else {
		for { set i [ expr $rpos + 1 ]  } { $i < $_rows } { incr i 1 } {
			clear_hrule $i
		}
	}

	return {}
}

body charoo::draw_vregion { cpos { direction "left" } } {
	if { $cpos > $_cols } {
		return "hrule too big for screen"
	}

	draw_vrule $cpos

	if { $direction == "left" } {
		for { set i [ expr $cpos - 1 ]  } { $i > 0 } { incr i -1 } {
			clear_vrule $i
		}
	} else {
		for { set i [ expr $cpos + 1 ]  } { $i < $_cols } { incr i 1 } {
			clear_vrule $i
		}
	}

	return {}
}

body charoo::clear_line_h { rpos cpos len } {

	set oe [ get_hedge ] 
	set_hedge $_clear
	
	draw_line_h $rpos $cpos $len

	set_hedge $oe

	return {}
}

body charoo::clear_line_v { rpos cpos len } {

	set oe [ get_vedge ] 
	set_vedge $_clear 

	draw_line_v $rpos $cpos $len

	set_vedge $oe

	return {}
}

body charoo::clear_hrule  { rpos } {

	set oe [ get_hedge ] 
	set_hedge $_clear
	
	draw_hrule $rpos 

	set_hedge $oe

	return {}
}

body charoo::clear_vrule  { cpos } {

	set oe [ get_vedge ] 
	set_vedge $_clear 

	draw_vrule $cpos

	set_vedge $oe

	return {}
}

proc cr_gettextboxsize { screen text } {
	
	set maxrlen 0 
	set currlen 1

	set rows 1

	set len [ string length $text ] 

	for { set i 0 } { $i < $len } { incr i } { 
		if { [ string index $text $i ] == "\n" } {
			incr rows

			if { $currlen > $maxrlen } {
				set maxrlen $currlen 
			}

			set currlen 1 
		} else {
			incr currlen
		}
	}

	if { $maxrlen == 0 } {
		set maxrlen $currlen
	}

	return [ list $rows $maxrlen ] 
}

body charoo::draw_textbox_c { text } {
	
	set xytlen [ cr_gettextboxsize $this $text ] 

	set rlen [ lindex $xytlen 0 ] 
	set clen [ lindex $xytlen 1 ] 

	set rlen [ incr rlen 2 ]
	set clen [ incr clen 1 ] 

	draw_blankbox_c $rlen $clen

	set rpos $_boxrpos
	set cpos $_boxcpos

	incr rpos 1
	incr cpos 1

	draw_pgraph $rpos $cpos $text

}

body charoo::draw_pgraph { rpos cpos text } {

	set xytlen [ cr_gettextboxsize $this $text ] 

	set rlen [ lindex $xytlen 0 ] 
	set clen [ lindex $xytlen 1 ] 

	term move $rpos $cpos

	for { set i 0 } { $i < $clen } { incr i } {
		set c [ string index $text $i ]

		if { $c == "\n" } {
			term move [ expr $rpos + [ expr $i + 1 ]  ] $cpos
		} else {
			term puts $c
		}
	}

	return {}
}

body charoo::draw_text { rpos cpos text } {

	set clen [ string length $text ] 

	term move $rpos $cpos

	for { set i 0 } { $i < $clen } { incr i } {
		term puts [ string index $text $i ] 
	}

	return {}
}

body charoo::draw_box { rpos cpos rsize csize } {
	set srow $rpos
	set scol $cpos

	set erow [ expr $srow + $rsize - 1 ] 
	set ecol [ expr $scol + $csize - 1 ] 

	set _boxrpos $srow
	set _boxcpos $scol

	draw_line_h $srow $scol $csize yes
	draw_line_h $erow $scol $csize yes

	draw_line_v $srow $scol $rsize yes
	draw_line_v $srow $ecol $rsize yes

	return {}
}

body charoo::draw_blankbox { rpos cpos rsize csize } { 

	set srow $rpos
	set scol $cpos

	set erow [ expr $srow + $rsize - 1 ] 
	set ecol [ expr $scol + $csize - 1 ] 

	set _boxrpos $srow
	set _boxcpos $scol

	draw_line_h $srow $scol $csize yes
	draw_line_h $erow $scol $csize yes

	draw_line_v $srow $scol $rsize yes
	draw_line_v $srow $ecol $rsize yes

	set srow [ incr srow 1 ] 
	set scol [ incr scol 1 ]

	set rsize [ incr rsize -2 ]
	set csize [ incr csize -2 ] 

	for { set i 0 } { $i < $rsize } { incr i } {
		clear_line_h $srow $scol $csize
		incr srow
	}

	return {}	

}

body charoo::draw_textbox { rpos cpos text } {

	set xytlen [ cr_gettextboxsize $this $text ] 

	set rlen [ lindex $xytlen 0 ] 
	set clen [ lindex $xytlen 1 ] 

	set rlen [ incr rlen 2 ]
	set clen [ incr clen 1 ] 

	draw_blankbox $rpos $cpos $rlen $clen

	set rpos $_boxrpos
	set cpos $_boxcpos

	incr rpos 1
	incr cpos 1

	draw_pgraph $rpos $cpos $text
}

body charoo::clear_box { rpos cpos rsize csize } {
	set srow $rpos
	set scol $cpos

	set erow [ expr $srow + $rsize - 1 ] 
	set ecol [ expr $scol + $csize - 1 ] 

	set _boxrpos $srow
	set _boxcpos $scol

	clear_line_h $srow $scol $csize 
	clear_line_h $erow $scol $csize 

	clear_line_v $srow $scol $rsize 
	clear_line_v $srow $ecol $rsize 

	return {}
}

body charoo::clear_blankbox { rpos cpos rsize csize } { 

	set srow $rpos
	set scol $cpos

	set erow [ expr $srow + $rsize - 1 ] 
	set ecol [ expr $scol + $csize - 1 ] 

	set _boxrpos $srow
	set _boxcpos $scol

	clear_line_h $srow $scol $csize 
	clear_line_h $erow $scol $csize 

	clear_line_v $srow $scol $rsize 
	clear_line_v $srow $ecol $rsize 

	set srow [ incr srow 1 ] 
	set scol [ incr scol 1 ]

	set rsize [ incr rsize -2 ]
	set csize [ incr csize -2 ] 

	for { set i 0 } { $i < $rsize } { incr i } {
		clear_line_h $srow $scol $csize
		incr srow
	}

	return {}	

}

body charoo::clear_textbox { rpos cpos text } {

	set xytlen [ cr_gettextboxsize $this $text ] 

	set rlen [ lindex $xytlen 0 ] 
	set clen [ lindex $xytlen 1 ] 

	set rlen [ incr rlen 2 ]
	set clen [ incr clen 1 ] 

	clear_blankbox $rpos $cpos $rlen $clen
}

body charoo::pause { } { 
	paint
	
	getchar 
}

body charoo::getchar { } { 
	return [ term getchar ]
}


body charoo::on_bold { } {
	term attr bold 0 0 
}

body charoo::off_bold { } {
	term attroff all
}

